@@ -13,7 +13,8 @@ from hachoir_parser import Parser
 from hachoir_core.field import (StaticFieldSet, FieldSet, Field,
     Bit, Bits,
     UInt32, UInt16, UInt8, Enum,
-    RawBytes, String, GenericVector, ParserError)
+    PaddingBytes, RawBytes, NullBytes,
+    String, GenericVector, ParserError)
 from hachoir_core.endian import LITTLE_ENDIAN
 from hachoir_core.text_handler import textHandler, hexadecimal
 from hachoir_core.tools import alignValue
@@ -36,9 +37,6 @@ class ChunkIndexer:
     def canHouse(self, chunk, index):
         if index > 1:
             if chunk.offset + chunk.size > self.chunks[index-1].offset:
-                print "Chunk: offset %u + size %u > next offset %u for '%s'!" % \
-                      (chunk.offset, chunk.size,
-                       self.chunks[index-1].offset, self.chunks[index-1].name)
                 return False
         # We could test now that it fits in the memory
         return True
@@ -52,15 +50,11 @@ class ChunkIndexer:
             if offset < new_chunk.offset:
                 if not self.canHouse(new_chunk, index):
                     raise ParserError("Chunk '%s' doesn't fit!" % new_chunk.name)
-                    ## print "Added element %u: size=%u position=%u>%u" % \
-                    ##       (index, new_chunk.size, new_chunk.offset, offset)
                 self.chunks.insert(index, new_chunk)
                 return
             index += 1
 
         # Not found or empty
-        ##print "Appended element %u of size %u at position %u" % \
-        ##      (len(self.chunks)+1, new_chunk.size, new_chunk.offset)
         # We could at least check that it fits in the memory
         self.chunks.append(new_chunk)
 
@@ -74,7 +68,7 @@ class ChunkIndexer:
             if size > 0:
                 obj.info("Padding of %u bytes needed: curr=%u offset=%u" % \
                          (size, current_pos, chunk.offset))
-                yield RawBytes(obj, "padding[]", size)
+                yield PaddingBytes(obj, "padding[]", size)
                 current_pos = obj.current_size//8
 
             # Find resynch point if needed
@@ -112,7 +106,6 @@ class ChunkIndexer:
 
             # Let missing padding be done by next chunk
 
-
 class S3MFlags(StaticFieldSet):
     format = (
         (Bit, "st2_vibrato", "Vibrato (File version 1/ScreamTrack 2)"),
@@ -194,7 +187,7 @@ class SizeFieldSet(FieldSet):
             yield field
         size = (self._size - self.current_size)//8
         if size > 0:
-            yield RawBytes(self, "padding", size)
+            yield PaddingBytes(self, "padding", size)
 
 class Header(SizeFieldSet):
     def createDescription(self):
@@ -272,18 +265,15 @@ class S3MHeader(Header):
         # Deduce size for SizeFieldSet
         self.setCheckedSizes(size)
 
-    def getTrackerVersion(val):
-        val = val.value
-        return "ScreamTracker %X.%02X" % ((val-0x1000)>>8, val&0xFF)
-
     def getFileVersionField(self):
         yield UInt8(self, "type")
         yield RawBytes(self, "reserved[]", 2)
 
     def getFirstProperties(self):
         yield S3MFlags(self, "flags")
-        yield textHandler(UInt16(self, "creation_version"),
-            S3MHeader.getTrackerVersion)
+        yield UInt8(self, "creation_version_minor")
+        yield Bits(self, "creation_version_major", 4)
+        yield Bits(self, "creation_version_unknown", 4, "(=1)")
         yield UInt16(self, "format_version")
 
     def getLastProperties(self):
@@ -310,14 +300,15 @@ class S3MHeader(Header):
             yield GenericVector(self, "pattern_pptr", patterns, UInt16, "offset")
 
         # S3M 3.20 extension
-        if self["creation_version"].value >= 0x1320 and \
-               self["panning_info"].value == 252:
+        if self["creation_version_major"].value >= 3 \
+        and self["creation_version_minor"].value >= 0x20 \
+        and self["panning_info"].value == 252:
             yield GenericVector(self, "channel_panning", 32, ChannelPanning, "channel")
 
         # Padding required for 16B alignment
         size = self._size - self.current_size
         if size > 0:
-            yield RawBytes(self, "padding", size//8)
+            yield PaddingBytes(self, "padding", size//8)
 
     def getSubChunks(self):
         # Instruments -  no warranty that they are concatenated
@@ -412,9 +403,14 @@ class Instrument(SizeFieldSet):
     static_size = 8*0x50
 
     def createDescription(self):
-        return "%s, %s, %ubits" % (self["c4_speed"].display, \
-                           ("mono", "stereo")[self["flags/stereo"].value],
-                           self.getSampleBits())
+        info = [self["c4_speed"].display]
+        if "flags/stereo" in self:
+            if self["flags/stereo"].value:
+                info.append("stereo")
+            else:
+                info.append("mono")
+        info.append("%u bits" % self.getSampleBits())
+        return ", ".join(info)
 
     # Structure knows its size and doesn't need padding anyway, so
     # overwrite base member: no need to go through it.
@@ -561,7 +557,6 @@ class PTMNoteInfo(StaticFieldSet):
 class Note(FieldSet):
     def createFields(self):
         # Used by Row to check if end of Row
-        self.header = self.stream.readBits(self.absolute_address, 8, LITTLE_ENDIAN)
         info = self.NOTE_INFO(self, "info")
         yield info
         if info["has_note"].value:
@@ -580,13 +575,17 @@ class PTMNote(Note):
 
 class Row(FieldSet):
     def createFields(self):
+        addr = self.absolute_address
         while True:
+            # Check empty note
+            byte = self.stream.readBits(addr, 8, self.endian)
+            if not byte:
+                yield NullBytes(self, "terminator", 1)
+                return
+
             note = self.NOTE(self, "note[]")
             yield note
-
-            # Check empty note
-            if note.header == 0:
-                break
+            addr += note.size
 
 class S3MRow(Row):
     NOTE = S3MNote
