@@ -88,6 +88,8 @@ testcase_files = (
     (u"angle-bear-48x48.ani", 29658, "9aa43520cf430bd996c4c318b905bc7f"),
     (u"hotel_california.flac", 32768, "ebd87fc310d8958c9d86ef65c7a20e1d"),
     (u"radpoor.doc", 103936, "114835a03be92e02029c74ece1162c3e"),
+    (u"quicktime.mp4", 245779, "dc77a8de8c091c19d86df74280f6feb7"),
+    (u"swat.blp", 55753, "a47a2d6ef61c9005c3f5faf1bca253af"),
 )
 
 def stringMD5(text):
@@ -185,7 +187,7 @@ def testFiles(directory, url_prefix):
                 return False
             if not checkFileMD5(fullname, md5sum):
                 return False
-        except OSError, err:
+        except OSError:
             # Error: file doesn't exist
             if not download(url_prefix+filename, filesize, md5sum, fullname):
                 return False
